#' @keywords internal
"_PACKAGE"

#' Defunct functions in shinyjs
#'
#' - **colourInput()** Moved to the \{colourpicker\} package
#' - **updateColourInput()** Moved to the \{colourpicker\} package
#' - **colourPicker()** Moved to the \{colourpicker\} package
#' - **runExample()** Use `shiny::runExample(package = "shinyjs")`
#'
#' @name shinyjs-defunct
NULL
