#' @rdname shinyjs-defunct
#' @export
#' @keywords internal
colourInput <- function(...) {
  .Defunct("colourInput", package = "colourpicker",
           msg = "colourInput() has been moved to the 'colourpicker' package.")
}

#' @rdname shinyjs-defunct
#' @export
#' @keywords internal
updateColourInput <- function(...) {
  .Defunct("updateColourInput", package = "colourpicker",
           msg = "updateColourInput() has been moved to the 'colourpicker' package.")
}

#' @rdname shinyjs-defunct
#' @export
#' @keywords internal
colourPicker <- function(...) {
  .Defunct("colourPicker", package = "colourpicker",
           msg = "colourPicker() has been moved to the 'colourpicker' package.")
}